﻿--[[
    Author: Valana <Team BATTLE> of Mug'thol Alliance-US

    Depends on TbChatEnhancements to suppress achievement messages from ignored players, channel join/leave messages, AFK/DND response
    messages, drunk messages, duel win announcements, and optionally to make sure the new user welcome message appears after
    TbChatEnhancements wipes the chat window.
]]

--
-- Must not be local: relied upon by TbCore:IsIgnored(name).
--
TbSilentIgnore = LibStub("AceAddon-3.0"):NewAddon("TbSilentIgnore", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TB_SilentIgnore", true)
local D = LibStub("AceConfigDialog-3.0")

local tinsert = tinsert
local sort = sort
local time = time
local format = format
local strupper = strupper
local strmatch = strmatch
local UnitName = UnitName

local _, _addon = GetAddOnInfo("TB_SilentIgnore")


-------------------------------------------------------------------------------
-- Override the /ignore command.
-------------------------------------------------------------------------------
TbSilentIgnore.Ignore = function(arg)
    if (TbCore:StrIsNilOrEmpty(arg)) then
        InterfaceOptionsFrame_OpenToCategory(D.BlizOptions[_addon].frame)
        return
    end

    --
    -- Extract the name and potential comment from the argument.
    --
    local name, comment = TbSilentIgnore:SplitIgnore(arg)
    if (TbSilentIgnore.db.realm.ignored[name] and TbCore:StrIsNilOrEmpty(comment)) then
        TbSilentIgnore.db.realm.ignored[name] = nil
        TbCore:PrintForChannel(format(ERR_IGNORE_REMOVED_S, name), "SYSTEM", "SYSTEM")
    else
        TbSilentIgnore.db.realm.ignored[name] = { ["time"] = time(), ["comment"] = comment }
        TbCore:PrintForChannel(format(ERR_IGNORE_ADDED_S, name), "SYSTEM", "SYSTEM")
    end
end


-------------------------------------------------------------------------------
-- Suppress chat messages to and from silently-ignored players.
-------------------------------------------------------------------------------
function TbSilentIgnore:InviteUnit(unit, ...)
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["INVITING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["InviteUnit"](unit, ...)
    end
end

function TbSilentIgnore:PARTY_INVITE_REQUEST(e, sender)
    if (TbCore:IsIgnored(sender)) then
        DeclineGroup()
        StaticPopup_Hide("PARTY_INVITE")
    end
end

function TbSilentIgnore:GuildInvite(unit, ...)
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["GUILD_INVITING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["GuildInvite"](unit, ...)
    end
end

function TbSilentIgnore:GUILD_INVITE_REQUEST(e, sender)
    if (TbCore:IsIgnored(sender)) then
        DeclineGuild()
        StaticPopup_Hide("GUILD_INVITE")
    end
end

function TbSilentIgnore:ArenaTeamInviteByName(size, unit, ...)
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["ARENA_INVITING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["ArenaTeamInviteByName"](size, unit, ...)
    end
end

function TbSilentIgnore:ARENA_TEAM_INVITE_REQUEST(e, sender, team)
    if (TbCore:IsIgnored(sender)) then
        DeclineArenaTeam()
        StaticPopup_Hide("ARENA_TEAM_INVITE")
    end
end

function TbSilentIgnore:OfferPetition(unit, ...)
    if (TbCore:StrIsNilOrEmpty(unit)) then
        unit = "target"
    end
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["SIGNATURE_REQUESTING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["OfferPetition"](unit, ...)
    end
end

function TbSilentIgnore:PETITION_SHOW(e)
    local _, _, _, _, source = GetPetitionInfo()
    if (TbCore:IsIgnored(source)) then
        ClosePetition()
    end
end

function TbSilentIgnore:CalendarEventInvite(unit, ...)
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["CALENDAR_EVENT_INVITING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["CalendarEventInvite"](unit, ...)
    end
end

function TbSilentIgnore:CALENDAR_OPEN_EVENT(e)
    --
    -- It appears the event information is not available immediately upon opening the event page, so wait a bit before performing our checks.
    --
    self:ScheduleTimer(
        function()
            --
            -- Go through all the event invitees and see if any of the moderators is not ignored.
            --
            for i = 1, CalendarEventGetNumInvites() do
                local name, _, _, _, _, moderator = CalendarEventGetInvite(i)
                if (TbCore:StrIsNotNilOrEmpty(moderator) and not TbCore:IsIgnored(name)) then return end
            end
            CalendarRemoveEvent()
        end, .5)
end

function TbSilentIgnore:StartDuel(unit, ...)
    if (TbCore:StrIsNilOrEmpty(unit)) then
        unit = "target"
    end
    if (TbCore:IsIgnored(unit)) then
        TbCore:PrintForChannel(format(L["DUEL_REQUESTING_IGNORED"], TbCore:Capitalize(UnitName(unit) or unit)), "SYSTEM", "SYSTEM")
    else
        self.hooks["StartDuel"](unit, ...)
    end
end

function TbSilentIgnore:DUEL_REQUESTED(e, sender)
    if (TbCore:IsIgnored(sender)) then
        CancelDuel()
        StaticPopup_Hide("DUEL_REQUESTED")
    end
end

function TbSilentIgnore:PlaceAuctionBid(kind, index, amount, ...)
    local _, _, _, _, _, _, _, _, _, _, _, owner =  GetAuctionItemInfo(kind, index)
    if (TbCore:IsIgnored(owner)) then
        TbCore:PrintForChannel(format(L["AUCTION_BIDDING_IGNORED"], owner), "SYSTEM", "SYSTEM")
    else
        return self.hooks["PlaceAuctionBid"](kind, index, amount, ...)
    end
end

function TbSilentIgnore:SendMailFrame_SendMail()
    local recipient = SendMailNameEditBox:GetText()
    if (TbCore:IsIgnored(recipient)) then
        TbCore:PrintForChannel(format(L["MAIL_SENDING_IGNORED"], TbCore:Capitalize(recipient)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["SendMailFrame_SendMail"]()
    end
end

function TbSilentIgnore:SendChatMessage(message, distribution, language, destination, ...)
    if (strupper(distribution) == "WHISPER" and TbCore:IsIgnored(destination)) then
        TbCore:PrintForChannel(format(L["WHISPERING_IGNORED"], TbCore:Capitalize(UnitName(destination) or destination)), "SYSTEM", "SYSTEM")
    else
        return self.hooks["SendChatMessage"](message, distribution, language, destination, ...)
    end
end

--
-- Suppress raid warnings from silently ignored players.
--
function TbSilentIgnore:RaidWarningFrame_OnEvent(frame, event, message, sender, ...)
	if (not (event == "CHAT_MSG_RAID_WARNING" and TbCore:IsIgnored(sender))) then
		return self.hooks[frame].OnEvent(frame, event, message, sender, ...)
	end
end

--
-- Suppress online/offline messages and /rand results from silently-ignored players.
--
TbSilentIgnore.SystemMessageFilter = function(frame, e, message, ...)
    if (TbCore:IsIgnored(strmatch(message, L["ONLINE"]))) then return true end
    if (TbCore:IsIgnored(strmatch(message, L["OFFLINE"]))) then return true end
    if (TbCore:IsIgnored(strmatch(message, L["RANDOM_ROLL_RESULT"]))) then return true end
end

TbSilentIgnore.LootMessageFilter = function(frame, e, message, ...)
    if (TbCore:IsIgnored(strmatch(message, L["LOOT_ITEM"]) or strmatch(message, L["CREATED_ITEM"]) or strmatch(message, L["LOOT_ROLL_ROLLED"]) or strmatch(message, L["LOOT_ROLL_GREED"]) or strmatch(message, L["LOOT_ROLL_NEED"]))) then return true end
end

TbSilentIgnore.ChatMessageFilter = function(frame, e, message, sender, ...)
    if (TbCore:IsIgnored(sender)) then return true end
end


-------------------------------------------------------------------------------
-- Initialize our addon.
-------------------------------------------------------------------------------
function TbSilentIgnore:OnInitialize()
    --
    -- Load our settings.
    --
    self.db = LibStub("AceDB-3.0"):New("TbSilentIgnoreDb",
    {
        global = { retention = 6, },
        realm =
        {
            ignored = { },
            loaded = { },
        }
    }, "Default")
    self.db:SetProfile("Default")

    --
    -- Remove outdated entries.
    --
    self:CleanUpIgnoreList()

    --
    -- Create our options menus.
    --
    LibStub("AceConfig-3.0"):RegisterOptionsTable(_addon,
        {
            type = "group",
            name = _addon,
            desc = GetAddOnMetadata("TB_SilentIgnore", L["DESCRIPTION_METADATA_FIELD"]),
            handler = TbSilentIgnore,
            args =
            {
                subHeader =
                {
                    order = 0,
                    type = "description",
                    name = L["MENU_SUBHEADER"],
                },
                convert =
                {
                    order = 10,
                    type = "execute",
                    name = L["MENU_CONVERT"],
                    desc = L["MENU_CONVERT_DESCRIPTION"],
                    func = "ConvertIgnoreList",
                },
                retentionSeparator =
                {
                    order = 11,
                    type = "description",
                    name = "",
                },
                retention =
                {
                    order = 20,
                    type = "range",
                    name = L["MENU_RETENTION"],
                    desc = L["MENU_RETENTION_DESCRIPTION"],
                    width = "full",
                    min = 0,
                    max = 12,
                    step = 1,
                    get = "GetRetention",
                    set = "SetRetention",
                },
                ignored =
                {
                    order = 30,
                    type = "input",
                    name = L["MENU_IGNORED"],
                    desc = L["MENU_IGNORED_DESCRIPTION"],
                    usage = L["MENU_IGNORED_USAGE"],
                    width = "full",
                    multiline = true,
                    -- pattern = "[%w+[,;%s]]+",
                    get = "GetIgnored",
                    set = "SetIgnored",
                },
            },
        })

    --
    -- Add our options menus to the Blizzard interface menu.
    --
    D:AddToBlizOptions(_addon)

    --
    -- Define the slash command.
    --
    self:RegisterChatCommand(strsub(SLASH_IGNORE1, 2), TbSilentIgnore.Ignore)
    self:RegisterChatCommand(strsub(SLASH_IGNORE2, 2), TbSilentIgnore.Ignore)
end

function TbSilentIgnore:OnEnable()
    --
    -- Replace the default ignore functions.
    --
    self:RawHook("AddIgnore", TbSilentIgnore.Ignore, true)
    self:RawHook("AddOrDelIgnore", TbSilentIgnore.Ignore, true)
    self:RawHook("DelIgnore", TbSilentIgnore.Ignore, true)

    --
    -- Block party invites to and from silently-ignored players.
    --
    self:RawHook("InviteUnit", "InviteUnit", true)
    self:RegisterEvent("PARTY_INVITE_REQUEST")

    --
    -- Block guild invites to and from silently-ignored players.
    --
    self:RawHook("GuildInvite", "GuildInvite", true)
    self:RegisterEvent("GUILD_INVITE_REQUEST")

    --
    -- Block arena team invites to and from silently-ignored players.
    --
    self:RawHook("ArenaTeamInviteByName", "ArenaTeamInviteByName", true)
    self:RegisterEvent("ARENA_TEAM_INVITE_REQUEST")

    --
    -- Block signature requests to and from silently-ignored players.
    --
    self:RawHook("OfferPetition", "OfferPetition", true)
    self:RegisterEvent("PETITION_SHOW")

    --
    -- Block calendar event invites to and from silently-ignored players.
    --
    self:RawHook("CalendarEventInvite", "CalendarEventInvite", true)
    self:RegisterEvent("CALENDAR_OPEN_EVENT")

    --
    -- Block duel requests to and from silently-ignored players.
    --
    self:RawHook("StartDuel", "StartDuel", true)
    self:RegisterEvent("DUEL_REQUESTED")

    --
    -- Block bids placed on auctions owned by silently-ignored players.
    --
    self:RawHook("PlaceAuctionBid", "PlaceAuctionBid", true)

    --
    -- Block outgoing mail to silently-ignored players.
    --
    self:RawHook("SendMailFrame_SendMail", "SendMailFrame_SendMail", true)

    --
    -- Block outgoing whispers to silently-ignored players.
    --
    self:RawHook("SendChatMessage", "SendChatMessage", true)

    --
    -- Hook the Raid Warning frame to block announcements from silently-ignored players.
    --
    self:RawHookScript(RaidWarningFrame, "OnEvent", "RaidWarningFrame_OnEvent")

    --
    -- Register to filter chat from silently-ignored players. TB_ChatEnhancements takes care of achievement messages, channel
    -- join/leave messages, and AFK/DND response messages, drunk messages, and duel win announcements.
    --
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", TbSilentIgnore.SystemMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", TbSilentIgnore.LootMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_IGNORED", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_TEXT_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_BATTLEGROUND", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_BATTLEGROUND_LEADER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_WHISPER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_SAY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", TbSilentIgnore.ChatMessageFilter)

    --
    -- Alert this player to the use of this add-on.
    --
    local name = UnitName("player")
    if (GetNumIgnores() > 0 or not self.db.realm.loaded[name]) then
        TbCore:Print(L["NEW_USER"])
        self.db.realm.loaded[name] = true
    end
end

function TbSilentIgnore:OnDisable()
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", TbSilentIgnore.SystemMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_LOOT", TbSilentIgnore.LootMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_IGNORED", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_GUILD", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_OFFICER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_TEXT_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SAY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_YELL", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BATTLEGROUND", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BATTLEGROUND_LEADER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_PARTY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_LEADER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_WARNING", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_EMOTE", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_WHISPER", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_SAY", TbSilentIgnore.ChatMessageFilter)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_YELL", TbSilentIgnore.ChatMessageFilter)
end


-------------------------------------------------------------------------------
-- Properties.
-------------------------------------------------------------------------------
local _sorted = { }
function TbSilentIgnore:GetIgnored()
    --
    -- Build the list of strings to display in the configuration menu, and sort it.
    --
    for name, data in pairs(self.db.realm.ignored) do
        local comment = ""
        if (TbCore:StrIsNotNilOrEmpty(data["comment"])) then
            comment = " (" .. data["comment"] .. ")"
        end
        tinsert(_sorted, name .. comment)
    end
    sort(_sorted)

    --
    -- Join all the strings, using the new line character as a separator.
    --
    local str = TbCore:StrJoin(_sorted, "\n")
    wipe(_sorted)
    return str
end

local _newList = { }
function TbSilentIgnore:SetIgnored(info, list, ...)
    --
    -- Break up the string using the new line character as a delimiter.
    --
    local delimiter = "\n"
    local index = strfind(list, delimiter)
    while (index) do
        --
        -- Add the first token to our array and remove it from our string.
        --
        if (index > 1) then
            local name, comment = self:SplitIgnore(strsub(list, 1, index - 1))
            if (TbCore:StrIsNotNilOrEmpty(name)) then
                _newList[name] = comment or ""
            end
        end
        list = TbCore:StrTrim(strsub(list, index + 1))
        index = strfind(list, delimiter)
    end

    --
    -- Add the last token to our table. Append a new line character to the end to facilitate the parsing.
    --
    if (TbCore:StrIsNotNilOrEmpty(list)) then
        local name, comment = self:SplitIgnore(list)
        if (TbCore:StrIsNotNilOrEmpty(name)) then
            _newList[name] = comment or ""
        end
    end

    --
    -- Remove from the ignore list those names which are not in the new one.
    --
    for name, data in pairs(self.db.realm.ignored) do
        if (not _newList[name]) then
            self.db.realm.ignored[name] = nil
        end
    end

    --
    -- Add to the ignore list those names which were not already in it, and update the comments attached to those which already were.
    --
    local t = time()
    for name, comment in pairs(_newList) do
        if (self.db.realm.ignored[name]) then
            if (TbCore:StrIsNotNilOrEmpty(comment)) then
                self.db.realm.ignored[name]["comment"] = comment
            end
        else
            self.db.realm.ignored[name] = { ["time"] = t }
        end
    end
end

function TbSilentIgnore:GetRetention()
    return self.db.global.retention
end

function TbSilentIgnore:SetRetention(info, value, ...)
    self.db.global.retention = value
end


-------------------------------------------------------------------------------
-- Miscellaneous helper function.
-------------------------------------------------------------------------------
function TbSilentIgnore:CleanUpIgnoreList()
    --
    -- Migrate the contents of the factionrealm database to the realm database.
    --
    if (self.db.factionrealm and self.db.factionrealm.ignored) then
        for name, data in pairs(self.db.factionrealm.ignored) do
            self.db.realm.ignored[name] = data
        end
        self.db.factionrealm.ignored = nil
        self.db.factionrealm.loaded = nil
    end

    local t = time()
    local retention = self:GetRetention() * 60 * 60 * 24 * 30
    for name, data in pairs(self.db.realm.ignored) do
        if (type(data) ~= "table") then
            --
            -- Upgrade entries from previous versions of TB Silent Ignore to include a time stamp of when they were added.
            -- Make it a table to support notes in the future.
            --
            self.db.realm.ignored[name] = { ["time"] = t }
        elseif (retention > 0 and t - data["time"] > retention) then
            --
            -- Delete entries older than 60 seconds * 60 minutes * 24 hours * 30 days * retention months.
            --
            self.db.realm.ignored[name] = nil
        end
    end
end

function TbSilentIgnore:ConvertIgnoreList()
    local t = time()
    for i = GetNumIgnores(), 1, -1 do
        local ignoreName = GetIgnoreName(i)
        if (not self.db.realm.ignored[ignoreName]) then
            self.db.realm.ignored[ignoreName] = { ["time"] = t }
        end
        self.hooks["DelIgnore"](ignoreName)
    end
end

function TbSilentIgnore:IsIgnored(name)
    name = name or ""
    name = UnitName(name) or name
    if (self.db.realm.ignored[name]) then
        --
        -- Update the last time this player was seen.
        --
        self.db.realm.ignored[name]["time"] = time()
        return true
    end

    --
    -- Try capitalizing the name.
    --
    name = TbCore:Capitalize(name)
    if (self.db.realm.ignored[name]) then
        --
        -- Update the last time this player was seen.
        --
        self.db.realm.ignored[name]["time"] = time()
        return true
    end
end

function TbSilentIgnore:SplitIgnore(arg)
    --
    -- Try to split the name in a Name (comment) format. Append a space character to facilitate parsing.
    --
    local name, comment = strmatch(arg .. " ", "([^%(%[{]+)[%(%[{]([^%)%]}]*)[%)%]}][^%)%]}]*")
    
    --
    -- If we didn't match anything, assume the name is the entire argument.
    --
    if (TbCore:StrIsNilOrEmpty(name)) then
        name = arg
        comment = nil
    elseif (TbCore:StrIsNilOrEmpty(comment)) then
        --
        -- The comment might be missing. Make sure it is nil, in case it was just white space.
        --
        comment = nil
    else
        comment = TbCore:StrTrim(comment)
    end
    return TbCore:Capitalize(name), comment
end
